package org.dromara.monitoring.mapper;


import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.dromara.monitoring.domain.MonEquipmentHistroy;
import org.dromara.monitoring.domain.MonEquipmentList;

import java.util.List;

@Mapper
    public interface MonitoringYsMapper {


     Integer setSunList();

    void insettList(@Param("list") List<MonEquipmentList> tempList);

    void insertHistroy(@Param("data") List<MonEquipmentHistroy> data);

    @Delete("TRUNCATE TABLE mon_equipment_list")
    void delList();

    List<MonEquipmentList> selDpList();

}
